using FitConnect;
using FitConnect.Encryption;
using FitConnect.Models;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;

namespace E2ETest;

[Order(30)]
[TestFixture]
public class StraightPreEncryptedForwardTest : EndToEndTestBase {
    private string _caseId = null!;
    private string _submissionId = null!;

    [Order(10)]
    [Test]
    public void SendingSubmission() {
        // Arrange
        var publicKey = (Sender as Sender).GetPublicKeyFromDestination(Settings.DestinationId)
            .Result;

        var joseEncryptor = new JoseEncryptor();
        var encryptedAttachment =
            joseEncryptor.Encrypt(publicKey, File.ReadAllBytes("Test.pdf"));

        var data = JsonConvert.SerializeObject(new {
            Name = "Encrypted Test",
            Content = "A simple test to upload pre-encrypted data"
        });

        var encryptedData = joseEncryptor.Encrypt(publicKey, data);

        var attachment = new Attachment("Test.pdf", "Simple Test");
        var metadata = FitConnect.Sender.CreateMetadata(new Submission() {
            Data = data,
            Attachments = new List<Attachment>() { attachment },
            ServiceType = new ServiceType() { Identifier = Settings.LeikaKey },
        });

        var encryptedMetadata = joseEncryptor.Encrypt(publicKey, metadata);


        // Act
        var submission = Sender.WithDestination(Settings.DestinationId)
            .WithServiceType("Straight forward test", Settings.LeikaKey)
            .WithEncryptedAttachments(new Dictionary<string, string>() {
                {
                    attachment.Id,
                    encryptedAttachment
                }
            })
            .WithEncryptedMetaData(encryptedMetadata)
            .WithEncryptedData(encryptedData)
            .Submit();

        _caseId = submission.CaseId!;
        _submissionId = submission.Id!;

        _caseId.Should().NotBeNull();
        _submissionId.Should().NotBeNull();
    }

    [Test]
    [Order(20)]
    public void Sender_GetSubmissionState() {
        // Act
        var status = Sender.GetStatusForSubmission(_caseId, Settings.DestinationId);

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(30)]
    public void Subscriber_GetSubmissionState() {
        // Act
        var status = Subscriber.GetStatusForSubmission(_caseId, Settings.DestinationId);

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(40)]
    public void RequestSubmission() {
        var subscriberWithSubmission = Subscriber.RequestSubmission(_submissionId);

        var data = subscriberWithSubmission.GetDataJson();
        Logger.LogInformation("Data {Data}", data);

        data.Should()
            .Contain("A simple test to upload pre-encrypted data");

        var attachments = subscriberWithSubmission.GetAttachments();
        attachments.First().Filename.Should().Be("Test.pdf");

        subscriberWithSubmission.AcceptSubmission();
    }

    [Test]
    [Order(50)]
    public void Sender_GetSubmissionState_AfterAccepting() {
        // Act
        var status = Sender.GetStatusForSubmission(_caseId, Settings.DestinationId);

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }
}
